// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <tuple>

class Math
{
public:
    /// <summary>
    /// Calculates an integer exponentiation.
    /// </summary>
    /// <param name="base">Base of the exponentiation.</param>
    /// <param name="exponent">Exponent of the exponentiation</param>
    /// <returns>'base' raised to the power of 'exponent'.</returns>
    static int ipow(const int base, const int exponent);

    /// <summary>
    /// Calculates the greatest common divisor gcd(a, b) and the coefficients x and y of Bézout's identity
    /// ax + by = gcd(a, b). If a and b are coprime, then x is the modular multiplicative inverse of a modulo b, and y
    /// is the modular multiplicative inverse of b modulo a.
    /// </summary>
    /// <param name="a"></param>
    /// <param name="b"></param>
    /// <returns>A tuple of the gcd(a, b), x, and y.</returns>
    static std::tuple<int, int, int> extendedEuclid(const int a, const int b);
};
